<?php defined("VALIDATOR") or die();

/**
 * ArchServ Validator
 * Copyright (C) 2013-2014  Bernhard R. Arnold
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

function highlight($line)
{
    $line = str_replace(" ", "<span class=\"space\"> </span>", $line);
    $line = str_replace("\t", "<span class=\"tab\">\t</span>", $line);
    return $line;
}

function count_lines($filename)
{
    $linecount = 0;
    $handle = fopen($filename, "r");
    while(!feof($handle)) {
        $line = fgets($handle);
        $linecount++;
    }

    fclose($handle);
    return $linecount;
}

function ndigits($value)
{
    return strlen("$value");
}

class Validator
{
    private $_pattern_loose = '/^([A-Z0-9]+)\s+(\-?\d+\.\d+)\s+(\-?\d+\.\d+)\s+(\-?\d+\.\d+)\s+([A-Z0-9]*)\s*\r?\n?$/';
    private $_pattern_strict = '/^([A-Z0-9]+)\t\t(\-?\d+\.\d+)\t(\-?\d+\.\d+)\t(\-?\d+\.\d+)(\t([A-Z0-9]*))?\t?\r?\n?$/';

    private function match_loose($line)
    {
        return preg_match($this->_pattern_loose, $line);
    }

    private function match_strict($line)
    {
        return preg_match($this->_pattern_strict, $line);
    }

    private function load_view($name, $attr = array())
    {
        extract($attr);
        include "system/views/$name.inc.php";
    }

    public function exec_()
    {

        $is_upload = $_SERVER['REQUEST_METHOD'] == 'POST';
        $filename = "";

        $report = "";
        $output = "";

        if ($is_upload) {

            // Undefined | Multiple Files | $_FILES Corruption Attack
            // If this request falls under any of them, treat it invalid.
            if (
                !isset($_FILES['file']['error']) ||
                is_array($_FILES['file']['error'])
            ) die("Invalid file parameters.");

            // Check $_FILES['upfile']['error'] value.
            switch ($_FILES['file']['error']) {
                case UPLOAD_ERR_OK:
                    break;
                case UPLOAD_ERR_NO_FILE:
                    $this->load_view("document", array("report" => "No file selected.", "output" => ""));
                    die();
                case UPLOAD_ERR_INI_SIZE:
                case UPLOAD_ERR_FORM_SIZE:
                    die("Exceeded filesize limit.");
                default:
                    die("Unknown errors.");
            }

            // You should also check filesize here.
            if ($_FILES['file']['size'] > 1000000)
                die("Exceeded filesize limit.");

            // DO NOT TRUST $_FILES['upfile']['mime'] VALUE !!
            // Check MIME Type by yourself.
            $finfo = new finfo(FILEINFO_MIME_TYPE);
            if (false === $ext = array_search(
            $finfo->file($_FILES['file']['tmp_name']),
            array(
                'asc' => 'text/plain',
                'txt' => 'text/plain',
                'dat' => 'text/plain',
            ),
            true
            )) die("Invalid file format.");

            $tmp_name = $_FILES['file']['tmp_name'];
            $filename = $_FILES['file']['name'];

            $total_lines = count_lines($tmp_name);

            $handle = fopen($tmp_name, 'r');
            $output = "";
            $line_nr = 1;
            $warnings = array();
            $errors = array();

            // this could be far better...
            while ($line = fgets($handle)) {
                $s = trim($line);
                $ptype = "point";
                if (strlen($s)) {
                    if (preg_match('/[A-Z]/', $s[0])) {
                        $ptype = "target";
                    }
                }
                $no = sprintf("<span style=\"color:#aaa;\"><a name=\"$line_nr\">%0".ndigits($total_lines)."s</a></span>", $line_nr);
                if ($this->match_loose($line)) {
                    $output .= $no.'<span class="'.$ptype.'"> </span><span style="background:#CCFF99;">'.highlight($line).'</span>';
                } else if ($this->match_strict($line)) {
                    $output .= $no.'<span class="'.$ptype.'"> </span><span style="background:#FFCC66;">'.highlight($line).'</span>';
                    $warnings[] = $line_nr;
                } else {
                    $output .= $no.'<span class="'.$ptype.'"> </span><span style="background:#FF5050;">'.highlight($line).'</span>';
                    $errors[] = $line_nr;
                }
                $line_nr++;
            }

            $report = "";

            if (sizeof($errors)) {
                $line_nr = sizeof($errors);
                $report = "<div>Encountered $line_nr errors in lines";
                foreach ($errors as $error) {
                    $report .= " <a href=\"#$error\">$error</a>";
                }
                $report .= "</div>";
            }

            if (sizeof($warnings)) {
                $line_nr = sizeof($warnings);
                $report .= "<div>Encountered $line_nr warnings in lines";
                foreach ($warnings as $warning) {
                    $report .= " <a href=\"#$warning\">$warning</a>";
                }
                $report .= "</div>";
            }

            if (!$report) $report = "<div>Validation successfull. No errors found.</div>";

            fclose($handle);

            unlink($tmp_name);

            $report = "<div>Validated $total_lines lines.</div>".$report;

        } // if post

        $this->load_view("document", array("report" => $report, "output" => $output));

    }
}
